
// NOTICE: 

//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.



#include "filter.h"
#include "resource.h"
#include <math.h>
#include <stdio.h>
#include <time.h>
#include <commctrl.h>




#define SCALE	16777216         /* scale factor: do floats with int math */
#define	SHIFT   24		   		 /* shift of scale 2^shift=scale */
#define MNNL 100					 /* Min Noise Luma */
#define MXNL 150				 /* Max Noise Luma */
//#define NTRESH 30				 /* Noise Threshold 1-200*/
#define SCRATCHES 3				 /* likely hood of scratches */


// ---- PROTOTYPES ----------

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, 
	int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

int filter01RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int startProc(FilterActivation *fa, const FilterFunctions *ff);
int configProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
int configProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void filter_string(const FilterActivation *fa, const FilterFunctions *ff, char *buf);



// ---- VARIABLES ----------

typedef struct MyFilterData {
	unsigned int	tab[3][3][256];    /* multiply lookup table */
	int y_offset;
	int scale[480];
	int sepiaR[256];
	int sepiaG[256];
	int noiseLU[256];
	int techno[256][256];
	int doStatic[1000];
	int mode;
	int lineNoise;
	int split;
	int noise;
	int noiseAmt;
	IFilterPreview *ifp;

} MyFilterData;


struct FilterDefinition filterDef_tutorial = {
    NULL, NULL, NULL,
    "colourLooks - 20050325",			// name
    "Some Funky Presets \n\r", // desc
    "trevlac",					// maker
    NULL,
    sizeof(MyFilterData),   // inst_data_size
	NULL, NULL,
    filter01RunProc,		    // runProc
    NULL,
	configProc,
	filter_string,
	startProc,					// init stuff
	NULL,
	NULL,NULL
};

static FilterDefinition *fd_tutorial;


// ---- FUNCTIONS ----------


//*** VirtualdubFilterModuleInit2 ****

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm,
 const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_tutorial = ff->addFilter(fm, &filterDef_tutorial, sizeof(FilterDefinition))))
        return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}


//*** VirtualdubFilterModuleDeinit ****

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm,
 const FilterFunctions *ff) {
    ff->removeFilter(fd_tutorial);
}

int randomNumber(int min_value, int max_value)  { 
    	
    double r;

    r = (  (double)rand() / ((double)(RAND_MAX)+(double)(1)) ); 
        return (int)(r * max_value - min_value) + min_value; 

}

//*** filter01RunProc ****

int filter01RunProc(const FilterActivation *fa, const FilterFunctions *ff) {
    PixDim w, h;
    Pixel *src, *dst, old_pixel;
	int luma, r, g, b, *myScale, cyan, newLuma, w2, noise, doNoise, *lineNoise, doLineNoise;
	double rd, gd, bd, mind;
	int NTRESH;

	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	NTRESH = mfd->noiseAmt;
	doNoise = mfd->noise;
	doLineNoise = mfd->lineNoise;
		


	lineNoise = (mfd->doStatic +  10) - randomNumber(5,10);;

	//doNoise = 1;

    src = fa->src.data;
    dst = fa->dst.data;

    h = fa->src.h;
	if (mfd->split==1) { w2 = fa->src.w/2;} else {w2=fa->src.w - 1;}

	switch (mfd->mode) {
	case 0:		//---- B&W --------

		do {w = fa->src.w;

			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {

				old_pixel = *src++;
				r = (old_pixel >> 16) & 0xff;
				g = (old_pixel>> 8) & 0xff;
				b = old_pixel & 0xff;
				luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

				//noise = randomNumber(MNNL,MXNL);
				noise = luma +(mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
				*dst = (randomNumber(0,200)<NTRESH)?
					((noise)<<16) + ((noise)<<8) + (noise):
					((luma)<<16) + ((luma)<<8) + (luma);

				*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);
			} else {

			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;
					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					*dst = ((luma)<<16) + ((luma)<<8) + (luma);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);
			}

			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;
			

	case 1:		//------ Sepia -------
		do {w = fa->src.w;
			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					//noise = randomNumber(MNNL,MXNL);
					noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
					*dst = (randomNumber(0,200)<NTRESH)?
						((noise + mfd->sepiaR[noise])<<16) + ((noise + mfd->sepiaG[noise])<<8) + (noise):
						((luma + mfd->sepiaR[luma])<<16) + ((luma + mfd->sepiaG[luma])<<8) + (luma);

					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;


			} while(--w);

			} else {

			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;
					*dst = ((luma + mfd->sepiaR[luma])<<16) + ((luma + mfd->sepiaG[luma])<<8) + (luma);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;

			} while(--w);
			}

			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;

	case 2:		//-- Technicolor --
		myScale = mfd->scale + 240;
		do {w = fa->src.w;
			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					//---- combine blue and green--
					//bd = sqrt(b*b + g*g) * (255.0/360.0);
					cyan = mfd->techno[b][g];

					//--- fudge them back out a bit ---
					newLuma = ((mfd->tab[0][0][(int)(r *.05)] + mfd->tab[0][1][(int)(g *.05)] + mfd->tab[0][2][(int)(b *.05)]) >> SHIFT) + mfd->y_offset;

					b = (int)(cyan - b *.05) + newLuma;
					g = (int)(cyan - g *.05) + newLuma;
					r = (int)(r  - r *.05) + newLuma;

					//noise = randomNumber(MNNL,MXNL);
					noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,1)-1)) ;
					*dst = (randomNumber(0,200)<NTRESH)?
						((noise)<<16) + ((noise)<<8) + (noise):
						((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;


			} while(--w);
			} else {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					//---- combine blue and green--
					//bd = sqrt(b*b + g*g) * (255.0/360.0);
					cyan = mfd->techno[b][g];

					//--- fudge them back out a bit ---
					newLuma = ((mfd->tab[0][0][(int)(r *.05)] + mfd->tab[0][1][(int)(g *.05)] + mfd->tab[0][2][(int)(b *.05)]) >> SHIFT) + mfd->y_offset;

					b = (int)(cyan - b *.05) + newLuma;
					g = (int)(cyan - g *.05) + newLuma;
					r = (int)(r  - r *.05) + newLuma;

					*dst = ((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;


			} while(--w);
			}

			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;


	case 3:		//---- POP --------

		do {w = fa->src.w;
			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					//if(luma < 200) {


					//---- un gamma correct --
					rd = pow(r/255.0,1.8) * 255.0;
					gd = pow(g/255.0,1.8) * 255.0;
					bd = pow(b/255.0,1.8) * 255.0;
					mind = min(min(rd,gd),bd);

					rd = rd-mind/2;
					gd = gd-mind/2;
					bd = bd-mind/2;

					//---- gamma correct --
					r = (int) (pow(rd/255.0,0.4545) * 255.0);
					g = (int) (pow(gd/255.0,0.4545) * 255.0);
					b = (int) (pow(bd/255.0,0.4545) * 255.0);
					//}

					noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
					*dst = (randomNumber(0,200)<NTRESH)?
						((noise)<<16) + ((noise)<<8) + (noise):
						((r)<<16) + ((g)<<8) + (b);

					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);

			} else {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					//luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					//if(luma < 200) {


					//---- un gamma correct --
					rd = pow(r/255.0,1.8) * 255.0;
					gd = pow(g/255.0,1.8) * 255.0;
					bd = pow(b/255.0,1.8) * 255.0;
					mind = min(min(rd,gd),bd);

					rd = rd-mind/2;
					gd = gd-mind/2;
					bd = bd-mind/2;

					//---- gamma correct --
					r = (int) (pow(rd/255.0,0.4545) * 255.0);
					g = (int) (pow(gd/255.0,0.4545) * 255.0);
					b = (int) (pow(bd/255.0,0.4545) * 255.0);
					//}

					*dst = ((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);
			}
			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;

		case 4:		//---- Bypass --------

		do {w = fa->src.w;

			if(doNoise) {
			do {*dst++ = *src++;} while((--w)-w2);
			do {

				old_pixel = *src;
				r = (old_pixel >> 16) & 0xff;
				g = (old_pixel>> 8) & 0xff;
				b = old_pixel & 0xff;

				luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

				//noise = randomNumber(MNNL,MXNL);
				noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
				*dst = ((randomNumber(0,200)<NTRESH)?
					((noise)<<16) + ((noise)<<8) + (noise):
					*src);
				*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
				src++;
			} while(--w);
			} else {

			do {*dst++ = *src++;} while(--w);
			}
			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;


		case 5:		//---- Cool --------

		do {w = fa->src.w;
			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;
					if((b -luma) < 0) {
						b=min(b+25,255);
					} else {
						r = max(r-25,0);
					}
					noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
					*dst = (randomNumber(0,200)<NTRESH)?
						((noise)<<16) + ((noise)<<8) + (noise):
						((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);
			
			} else {

			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;
					if((b -luma) < 0) {
						b=min(b+25,255);
					} else {
						r = max(r-25,0);
					}
					*dst = ((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);
			}

			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;
			
		case 6:		//---- Warm --------

		do {w = fa->src.w;
			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;
					if((r -luma) < 0) {
						r=min(r+19,255);
					} else {
						b = max(b-19,0);
						g = max(g-19,0);
					}
					noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
					*dst = (randomNumber(0,200)<NTRESH)?
						((noise)<<16) + ((noise)<<8) + (noise):
						((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);

			} else {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;
					if((r -luma) < 0) {
						r=min(r+19,255);
					} else {
						b = max(b-19,0);
						g = max(g-19,0);
					}
					*dst = ((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;
					
			} while(--w);
			}

			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;

		
		case 7:		//---- Matrix --------

		do {w = fa->src.w;
			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					if(g>r && g>b) {
						g=min(g+15,255);
					} else {
						b = max(b-15,0);
						r = max(r-15,0);
					}
					noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
					*dst = (randomNumber(0,200)<NTRESH)?
						((noise)<<16) + ((noise)<<8) + (noise):
						((r)<<16) + ((g)<<8) + (b);

					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;

			} while(--w);

			} else {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					if(g>r && g>b) {
						g=min(g+15,255);
					} else {
						b = max(b-15,0);
						r = max(r-15,0);
					}
					*dst = ((r)<<16) + ((g)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;

			} while(--w);
			}

			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;


		case 8:		//---- Loadus --------

		do {w = fa->src.w;
			do {*dst++ = *src++;} while((--w)-w2);

			if(doNoise) {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					luma =  ((mfd->tab[0][0][r] + mfd->tab[0][1][g] + mfd->tab[0][2][b]) >> SHIFT) + mfd->y_offset;

					//noise = randomNumber(MNNL,MXNL);
					noise = luma + (mfd->noiseLU[luma] * (randomNumber(0,2)-1)) ;
					*dst = (randomNumber(0,200)<NTRESH)?
						((noise)<<16) + ((noise)<<8) + (noise):
						((r)<<16) + (((g+b)>>1)<<8) + (b);

					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;

			} while(--w);
			} else {
			do {
					old_pixel = *src++;
					r = (old_pixel >> 16) & 0xff;
					g = (old_pixel>> 8) & 0xff;
					b = old_pixel & 0xff;

					*dst = ((r)<<16) + (((g+b)>>1)<<8) + (b);
					*dst++ = (doLineNoise)?*dst & lineNoise[w]:*dst;

			} while(--w);
			}
			src = (Pixel *)((char *)src + fa->src.modulo);
			dst = (Pixel *)((char *)dst + fa->dst.modulo);
		} while(--h);
		break;
	
	}


    return 0;
}





int startProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    srand ( static_cast <unsigned> (time(0)) ); 

	unsigned int	f;
	int	pv, j, b, g;
	//double y_matrix[3] = { 0.299, 0.587, 0.114,};	 //** 601 0-255
	double y_matrix[3] = { 0.2126, 0.7152, 0.0722,}; //** 709 0-255

	//mfd->y_offset = 16;
	//y_matrix[0] = 0.2567890625;
	//y_matrix[1] = 0.50412890625;
	//y_matrix[2] = 0.0979625;


	mfd->scale[0] = 0;
	for(j = 1; j<474; j++) 	mfd->scale[j+3] = (int)((j+3) * (255.0/474.0) + 0.5) ;

	//--- Sepia Lookup --
	for(j = 0; j<128; j++) {
		mfd->sepiaR[j] = (int) (j * 0.61);
		mfd->sepiaG[j] = (int) (j * 0.37);
		mfd->sepiaR[255-j] = mfd->sepiaR[j];
		mfd->sepiaG[255-j] = mfd->sepiaG[j];
	}

	//--- noise Lookup --
	for(j = 0; j<128; j++) {
		mfd->noiseLU[j] = (int) (j * 0.30);
		mfd->noiseLU[255-j] = mfd->noiseLU[j];
	}

	//--- Techno Lookup --
	for(b = 0; b<256; b++) {
			for(g = 0; g<256; g++) {
					mfd->techno[b][g] = (int) (sqrt(b*b + g*g) * (255.0/360.0));
			}
	}


	//--- Luma Lookup --
	for (pv = 0; pv <= 255; pv++) {
		f = SCALE * (pv );
		mfd->tab[0][0][pv] = (unsigned int)(f * (y_matrix[0]) + 0.5);
		mfd->tab[0][1][pv] = (unsigned int)(f * (y_matrix[1]) + 0.5);
		mfd->tab[0][2][pv] = (unsigned int)(f * (y_matrix[2]) + 0.5);
	}

	//for(j=0;j<1000;j++) {
	//	mfd->doStatic[j] = (randomNumber(0,200)<5)?0x777777:0xFFFFFF;
	//}

    return 0;
}

void calcScratches(MyFilterData *mfd) {

	for(int j=0;j<1000;j++) {
		mfd->doStatic[j] = (randomNumber(0,200)<SCRATCHES)?0x777777:0xFFFFFF;
	}


}


BOOL CALLBACK configDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);
	HWND hWnd;

    switch(msg) {
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;
            CheckDlgButton(hdlg, IDC_BW, mfd->mode==0?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_SEPIA, mfd->mode==1?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_TECH, mfd->mode==2?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_POP, mfd->mode==3?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_BYPASS, mfd->mode==4?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_COOL, mfd->mode==5?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_WARM, mfd->mode==6?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_MATRIX, mfd->mode==7?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_LOADUS, mfd->mode==8?BST_CHECKED:BST_UNCHECKED);

            CheckDlgButton(hdlg, IDC_SPLIT, mfd->split==1?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_NOISE, mfd->noise==1?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_LNOISE, mfd->lineNoise==1?BST_CHECKED:BST_UNCHECKED);


			hWnd = GetDlgItem(hdlg, IDC_NOISEAMT);
			SendMessage(hWnd, TBM_SETRANGE, (WPARAM)TRUE, MAKELONG(10, 100));
			SendMessage(hWnd, TBM_SETTICFREQ, 5 , 0);
			SendMessage(hWnd, TBM_SETPOS, (WPARAM)TRUE, mfd->noiseAmt);
		

			if (mfd->ifp)
				mfd->ifp->InitButton(GetDlgItem(hdlg, IDC_PREVIEW));


            return TRUE;

		case WM_HSCROLL:
			mfd->noiseAmt = SendMessage(GetDlgItem(hdlg, IDC_NOISEAMT), TBM_GETPOS, 0, 0);
			mfd->ifp->RedoFrame();
            return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
                if(IsDlgButtonChecked(hdlg, IDC_BW)) mfd->mode=0;
                if(IsDlgButtonChecked(hdlg, IDC_SEPIA)) mfd->mode=1;
                if(IsDlgButtonChecked(hdlg, IDC_TECH)) mfd->mode=2;
                if(IsDlgButtonChecked(hdlg, IDC_POP)) mfd->mode=3;
                if(IsDlgButtonChecked(hdlg, IDC_BYPASS)) mfd->mode=4;
                if(IsDlgButtonChecked(hdlg, IDC_COOL)) mfd->mode=5;
                if(IsDlgButtonChecked(hdlg, IDC_WARM)) mfd->mode=6;
                if(IsDlgButtonChecked(hdlg, IDC_MATRIX)) mfd->mode=7;
                if(IsDlgButtonChecked(hdlg, IDC_LOADUS)) mfd->mode=8;
                EndDialog(hdlg, 0);
                return TRUE;
            case IDCANCEL:
                EndDialog(hdlg, 1);
                return FALSE;

			case IDC_PREVIEW:
				if (mfd->ifp)
					mfd->ifp->Toggle(hdlg);
				return TRUE;

			case IDC_DARK:
			case IDC_BRIGHT:
			case IDC_GAMMA:
			case IDC_SCALE:
			case IDC_BYPASS:
			case IDC_COOL:
			case IDC_WARM:
			case IDC_MATRIX:
			case IDC_SPLIT:
			case IDC_NOISE:
			case IDC_LOADUS:
                if(IsDlgButtonChecked(hdlg, IDC_BW)) mfd->mode=0;
                if(IsDlgButtonChecked(hdlg, IDC_SEPIA)) mfd->mode=1;
                if(IsDlgButtonChecked(hdlg, IDC_TECH)) mfd->mode=2;
                if(IsDlgButtonChecked(hdlg, IDC_POP)) mfd->mode=3;
                if(IsDlgButtonChecked(hdlg, IDC_BYPASS)) mfd->mode=4;
                if(IsDlgButtonChecked(hdlg, IDC_COOL)) mfd->mode=5;
                if(IsDlgButtonChecked(hdlg, IDC_WARM)) mfd->mode=6;
                if(IsDlgButtonChecked(hdlg, IDC_MATRIX)) mfd->mode=7;
                if(IsDlgButtonChecked(hdlg, IDC_LOADUS)) mfd->mode=8;
                mfd->split = !!(IsDlgButtonChecked(hdlg,IDC_SPLIT));
                mfd->noise = !!(IsDlgButtonChecked(hdlg,IDC_NOISE));

				if (mfd->ifp) mfd->ifp->RedoFrame();

				return TRUE;

			case IDC_LNOISE:
                mfd->lineNoise = !!(IsDlgButtonChecked(hdlg,IDC_LNOISE));
				if(mfd->lineNoise) calcScratches(mfd);
				if (mfd->ifp) mfd->ifp->RedoFrame();

				return TRUE;


            }
            break;
    }

    return FALSE;
}

int configProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {

	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	mfd->ifp = fa->ifp;


    return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER), hwnd,
            configDlgProc, (LPARAM)fa->filter_data);
}


void filter_string(const FilterActivation *fa, const FilterFunctions *ff, char *buf) {

	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	switch (mfd->mode) {
	case 0:		//---- B&W --------
		sprintf(buf, " - B&W" );
		break;
	case 1:		//---- Sepia --------
		sprintf(buf,  " - Sepia" );
		break;
	case 2:		//---- Techni --------
		sprintf(buf,  " - Technicolor" );
		break;
	case 3:		//---- Pop --------
		sprintf(buf,  " - Pop" );
		break;
	case 4:		//---- Bypass --------
		sprintf(buf,  " - Bypass" );
		break;
	case 5:		//---- Bypass --------
		sprintf(buf,  " - Cool" );
		break;
	case 6:		//---- Bypass --------
		sprintf(buf,  " - Warm" );
		break;
	case 7:		//---- Bypass --------
		sprintf(buf,  " - Matrix" );
		break;
	case 8:		//---- Bypass --------
		sprintf(buf,  " - Loadus" );
		break;
	}

}

